/*
 * Copyright (c) 2017, Red Hat Inc. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */
package org.openjdk.gcbench;

import org.openjdk.gcbench.util.Dummy;
import org.openjdk.jmh.runner.Runner;
import org.openjdk.jmh.runner.RunnerException;
import org.openjdk.jmh.runner.options.Options;
import org.openjdk.jmh.runner.options.OptionsBuilder;
import org.openjdk.jmh.runner.options.VerboseMode;

import java.io.PrintWriter;

public class HeapSizeManager {

    public static int MAX_HEAP;
    public static int MIN_HEAP;

    public static void override(int min, int max) {
        MIN_HEAP = min;
        MAX_HEAP = max;
    }

    public static void init(int min, PrintWriter pw) {
        int baseHeapMB = 1000;
        int latestSuccessMB = 0;
        boolean progress;
        do {
            progress = false;
            for (int incr = 1000; incr < Integer.MAX_VALUE; incr *= 2) {
                int heapGB = baseHeapMB + incr;
                pw.print(heapGB + "? ");
                pw.flush();
                Options opts = new OptionsBuilder()
                        .include(Dummy.class.getCanonicalName())
                        .threads(1)
                        .jvmArgsAppend("-Xmx" + heapGB + "m", "-Xms" + heapGB + "m", "-XX:+AlwaysPreTouch")
                        .verbosity(VerboseMode.SILENT)
                        .build();
                try {
                    new Runner(opts).runSingle();
                    latestSuccessMB = heapGB;
                    progress = true;
                } catch (RunnerException e) {
                    baseHeapMB = latestSuccessMB;
                    break;
                }
            }
        } while (progress);

        pw.println();
        pw.println("Max heap size is " + latestSuccessMB + " Mb");
        pw.println();

        MIN_HEAP = min;
        MAX_HEAP = latestSuccessMB;
    }

}
